package kge.core;

import haxe.Constraints.Function;
import kge.math.Rectangle;
import kge.utils.XMLParser;
import kha.Assets;
import kha.Blob;
import kha.Color;
import kha.Image;
import kha.graphics2.GraphicsExtension;

class Graphic extends Basic
{
	public var width(get, set):Float;
	public var height(get, set):Float;
	private var _width:Float;
	private var _height:Float;
	
	public var color:Color;
	
	private var drawingFunction:Function;

	public function new(x:Float, y:Float, width:Float, height:Float) 
	{
		super(x, y);
		
		this._width = width;
		this._height = height;
		
		color = Color.White;
		
		setRectangle(width, height);
	}
	
	override public function render(framebuffer:Image) 
	{
		super.render(framebuffer);
		
		#if debug
		Debug.drawCalls++;
		
		if (Debug.drawBounds) {
			//TODO
		}
		#end
		
		framebuffer.g2.pushTransformation(framebuffer.g2.transformation.multmat(transform));
		
		framebuffer.g2.color = color;
		drawingFunction(framebuffer);
		
		framebuffer.g2.popTransformation();
	}
	
	public function setRectangle(width:Float, height:Float) {
		drawingFunction = drawRectangle.bind(_, width, height);
	}
	
	public function setCircle(radius:Float, segments:Int = 0) {
		drawingFunction = drawCircle.bind(_, radius, segments);
	}
	
	public function setImage(image:Image) {
		drawingFunction = drawImage.bind(_, image);
	}
	
	public function setAtlasImage(image:Image, xmlData:Blob, frameName:String) {
		this.drawingFunction = drawSubImage.bind(_, image, XMLParser.parseXMLAtlas(xmlData).frames.get(frameName).rectangle);
	}
	
	public function setCustomDrawFunction(drawingFunction:Function) {
		this.drawingFunction = drawingFunction;
	}
	
	private function drawRectangle(framebuffer:Image, width:Float, height:Float) {
		framebuffer.g2.fillRect(0, 0, width, height);
	}
	
	private function drawCircle(framebuffer:Image, radius:Float, segments:Int = 0) {
		GraphicsExtension.fillCircle(framebuffer.g2, 0, 0, radius, segments);
	}
	
	private function drawImage(framebuffer:Image, image:Image) {
		framebuffer.g2.drawImage(image, 0, 0);
	}
	
	private function drawSubImage(framebuffer:Image, image:Image, subRect:Rectangle) {
		framebuffer.g2.drawSubImage(image, 0, 0, subRect.x, subRect.y, subRect.width, subRect.height);
	}
	
	private function get_width():Float {
		return _width * scale.x;
	}
	private function set_width(value:Float):Float {
		return _width = value;
	}
	
	private function get_height():Float {
		return _height * scale.y;
	}
	private function set_height(value:Float):Float {
		return _height = value;
	}
}