package kge.core;

import kha.input.Mouse;
import kha.math.Vector2;
import de.polygonal.ds.LinkedQueue;

class MouseInput extends CoreBase
{
	private var buttonData:Map<Int, InputState>;
	private var pressedQueue:LinkedQueue<Int>;
	private var releasedQueue:LinkedQueue<Int>;
	
	public var x(get, null):Float;
	public var y(get, null):Float;
	public var mousePosition(get, null):Vector2;
	public var mousePosDelta(get, null):Vector2;
	private var _mousePosition:Vector2;
	private var _mousePosDelta:Vector2;
	
	public var onMouseDown:Signal = new Signal();
	public var onMouseUp:Signal = new Signal();
	public var onMouseMove:Signal = new Signal();
	public var onMouseWheel:Signal = new Signal();
	
	public var mouseWheel(get, null):Int;

	public function new() 
	{
		super();
		
		_mousePosition = new Vector2();
		_mousePosDelta = new Vector2();
		
		buttonData = new Map();
		pressedQueue = new LinkedQueue(5);
		releasedQueue = new LinkedQueue(5);
		
		Mouse.get().notify(mouseDownListener, mouseUpListener, mouseMoveListener, mouseWheelListener);
	}
	
	private function mouseDownListener(index:Int, x:Int, y:Int) {
		_mousePosition.x = x;
		_mousePosition.y = y;
		_mousePosDelta.x = 0;
		_mousePosDelta.y = 0;
		buttonData.set(index, InputState.PRESSED);
		pressedQueue.enqueue(index);
		onMouseDown.dispatch([index, x, y]);
	}
	
	private function mouseUpListener(index:Int, x:Int, y:Int) {
		_mousePosition.x = x;
		_mousePosition.y = y;
		_mousePosDelta.x = 0;
		_mousePosDelta.y = 0;
		buttonData.set(index, InputState.RELEASED);
		releasedQueue.enqueue(index);
		onMouseUp.dispatch([index, x, y]);
	}
	
	private function mouseMoveListener(x:Int, y:Int, dx:Int, dy:Int) {
		_mousePosition.x = x;
		_mousePosition.y = y;
		_mousePosDelta.x = dx;
		_mousePosDelta.y = dy;
		onMouseMove.dispatch([x, y, dx, dy]);
	}
	
	private function mouseWheelListener(delta:Int) {
		mouseWheel = delta;
		onMouseWheel.dispatch([delta]);
	}
	
	public function buttonDown(buttonValue:Int):Bool {
		var state:InputState = buttonData.get(buttonValue);
		return state == InputState.DOWN || state == InputState.PRESSED;
	}
	
	public function buttonUp(buttonValue:Int):Bool {
		var state:InputState = buttonData.get(buttonValue);
		return state == InputState.UP || state == InputState.RELEASED;
	}
	
	public function buttonPressed(buttonValue:Int):Bool {
		return buttonData.get(buttonValue) == InputState.PRESSED;
	}
	
	public function buttonReleased(buttonValue:Int):Bool {
		return buttonData.get(buttonValue) == InputState.RELEASED;
	}
	
	override public function update() {
		super.update();
		
		checkQueue(releasedQueue, UP);
		checkQueue(pressedQueue, DOWN);
		
		mouseWheel = 0;
	}
	
	private function checkQueue(queue:LinkedQueue<Int>, state:InputState) {
		var key:String;
		while (!queue.isEmpty()) {
			var key = queue.dequeue();
			if (buttonData.exists(key)) {
				buttonData.set(key, state);
			}
		}
	}
	
	public function get_x():Float {
		return _mousePosition.x / Game.scaleX;
	}

	public function get_y():Float {
		return _mousePosition.y / Game.scaleY;
	}

	public function get_mousePosition():Vector2 {
		return new Vector2(_mousePosition.x / Game.scaleX, _mousePosition.y / Game.scaleY);
	}

	public function get_mousePosDelta():Vector2 {
		return new Vector2(_mousePosDelta.x / Game.scaleX, _mousePosDelta.y / Game.scaleY);
	}
	
	public function get_mouseWheel():Int {
		return this.mouseWheel;
	}
}